import 'package:flutter/widgets.dart';
import 'package:yuro_core/yuro_core.dart';

import 'binding.dart';
import '../yuro_page.dart';

abstract class Middleware {
  Middleware(this.priority) : assert(priority >= 0 && priority <= 100);

  final int priority;

  RouteSettings redirect(String route);

  YuroPage onPageCalled(YuroPage page);

  Binding onBindingStart(Binding binding);

  YuroPageBuild onPageBuildStart(YuroPageBuild build);

  Widget onPageBuildEnd(Widget widget);

  void onPageDispose();
}

class YuroMiddleware extends Middleware {
  YuroMiddleware({int priority = 50}) : super(priority);

  @override
  RouteSettings redirect(String route) => null;

  @override
  YuroPage onPageCalled(YuroPage page) => page;

  @override
  Binding onBindingStart(Binding binding) => binding;

  @override
  YuroPageBuild onPageBuildStart(YuroPageBuild build) => build;

  @override
  Widget onPageBuildEnd(Widget widget) => widget;

  @override
  void onPageDispose() {}
}

/// runRedirect => runPageCalled => runBindingStart => runPageBuildStart => runPageBuildEnd => runPageDispose
class MiddlewareRunner {
  MiddlewareRunner(this.middlewares);

  final List<Middleware> middlewares;

  List<Middleware> _getMiddlewareList() {
    if (middlewares != null && middlewares.length > 0) {
      middlewares.sort((a, b) => a.priority.compareTo(b.priority));
    }
    return <Middleware>[];
  }

  RouteSettings runRedirect(String route) {
    RouteSettings settings;
    _getMiddlewareList().forEach((element) {
      settings = element.redirect(route);
    });
    if (settings != null) Yuro.log('重定向=>$settings');
    return settings;
  }

  YuroPage runPageCalled(YuroPage page) {
    _getMiddlewareList().forEach((element) {
      page = element.onPageCalled(page);
    });
    return page;
  }

  Binding runBindingStart(Binding binding) {
    _getMiddlewareList().forEach((element) {
      binding = element.onBindingStart(binding);
    });
    return binding;
  }

  YuroPageBuild runPageBuildStart(YuroPageBuild build) {
    _getMiddlewareList().forEach((element) {
      build = element.onPageBuildStart(build);
    });
    return build;
  }

  Widget runPageBuildEnd(Widget widget) {
    _getMiddlewareList().forEach((element) {
      widget = element.onPageBuildEnd(widget);
    });
    return widget;
  }

  void runPageDispose() => _getMiddlewareList().forEach((element) {
        element.onPageDispose();
      });
}
