import 'package:flutter/widgets.dart';

import '../yuro_page.dart';

class RouteTree {
  final _nodes = <_RouteTreeNode>[];

  void addRoute(YuroPage page) {
    var path = page.name;
    // 根路径
    if (path == Navigator.defaultRouteName) {
      var _node = _RouteTreeNode(path, _RouteTreeNodeType.Component)..page = page;
      _nodes.add(_node);
      return;
    }
    // 多级路径, /a/b/c
    if (path.startsWith('/')) path = path.substring(1);
    var paths = path.split('/');
    _RouteTreeNode parent;
    for (int i = 0; i < paths.length; i++) {
      final path = paths[i];
      var node = _nodeForPath(path, parent);
      if (node == null) {
        node = _RouteTreeNode(path, _nodeType(path))..parent = parent;
        if (parent == null) {
          _nodes.add(node);
        } else {
          parent.children.add(node);
        }
      }
      // 最后一级,存入page
      if (i == paths.length - 1) node.page = page;
      parent = node;
    }
  }

  _RouteTreeNode _nodeForPath(String path, _RouteTreeNode parent) {
    var nodes = parent == null ? _nodes : parent.children;
    var matchNodes = nodes.where((element) => path == element.path).toList();
    return matchNodes.isEmpty ? null : matchNodes[0];
  }

  _RouteTreeNodeType _nodeType(String path) {
    var type = _RouteTreeNodeType.Component;
    if (path.startsWith(':')) {
      type = _RouteTreeNodeType.Parameter;
    }
    return type;
  }

  RouteMatch matchRoute(String path) {
    var queryMap = <String, String>{};
    List<String> paths;
    if (path == Navigator.defaultRouteName) {
      paths = ['/'];
    } else {
      var uri = Uri.tryParse(path);
      paths = uri.pathSegments;
      queryMap.addAll(uri.queryParameters);
    }

    var nodesToCheck = _nodes;
    var nodesMatch = <_RouteTreeNode, _RouteTreeNodeMatch>{};
    paths.forEach((path) {
      final currentMatch = <_RouteTreeNode, _RouteTreeNodeMatch>{};
      final nextNodes = <_RouteTreeNode>[];
      nodesToCheck.forEach((node) {
        final isMatch = path == node.path || node.isParameter;
        if (isMatch) {
          final parentMatch = nodesMatch[node.parent];
          final match = _RouteTreeNodeMatch.fromMatch(parentMatch, node);
          if (node.isParameter) {
            match.parameters[node.path.substring(1)] = path;
          }
          if (queryMap.isNotEmpty) {
            match.parameters.addAll(queryMap);
          }
          currentMatch[node] = match;
          if (node.children.isNotEmpty) {
            nextNodes.addAll(node.children);
          }
        }
      });

      nodesMatch = currentMatch;
      nodesToCheck = nextNodes;
    });

    var matches = nodesMatch.values.toList();
    if (matches.isNotEmpty) {
      var pathMatches = matches.where((element) => !element.node.isParameter).toList();
      var match = pathMatches.length > 0 ? pathMatches[0] : matches.first;
      var node = match.node;

      if (node != null && node.page != null) {
        return RouteMatch(node.page)..parameters.addAll(match.parameters);
      }
    }
    return null;
  }
}

class _RouteTreeNode {
  _RouteTreeNode(this.path, this.type);

  final String path;
  final _RouteTreeNodeType type;

  YuroPage page;
  _RouteTreeNode parent;
  List<_RouteTreeNode> children = [];

  bool get isParameter => type == _RouteTreeNodeType.Parameter;
}

class _RouteTreeNodeMatch {
  _RouteTreeNodeMatch(this.node);

  _RouteTreeNodeMatch.fromMatch(_RouteTreeNodeMatch match, this.node) {
    if (match != null) parameters.addAll(match.parameters);
  }

  final _RouteTreeNode node;
  Map<String, String> parameters = {};
}

enum _RouteTreeNodeType { Component, Parameter }

class RouteMatch {
  RouteMatch(this.page);

  final YuroPage page;

  Map<String, String> parameters = {};
}
