import 'package:flutter/material.dart';

enum Transition {
  FADE,
  FADE_IN,
  RIGHT_TO_LEFT,
  LEFT_TO_RIGHT,
  UP_TO_DOWN,
  DOWN_TO_UP,
  RIGHT_TO_LEFT_WITH_FADE,
  LEFT_TO_RIGHT_WITH_FADE,
  ZOOM,
  TOP_LEVEL,
  NO_TRANSITION,
  CUPERTINO,
  // CUPERTINO_DIALOG,
  SIZE,
  NATIVE
}

abstract class CustomTransition {
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  );
}

class NoTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      child;
}

class FadeInTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      FadeTransition(opacity: animation, child: child);
}

class LeftToRightTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      SlideTransition(
        position: Tween<Offset>(begin: Offset(-1, 0), end: Offset.zero).animate(animation),
        child: child,
      );
}

class RightToLeftTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      SlideTransition(
        position: Tween<Offset>(begin: Offset(1, 0), end: Offset.zero).animate(animation),
        child: child,
      );
}

class LeftToRightWithFadeTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      SlideTransition(
          position: Tween<Offset>(begin: Offset(-1, 0), end: Offset.zero).animate(animation),
          child: FadeTransition(
              opacity: animation,
              child: SlideTransition(
                position: Tween<Offset>(begin: Offset.zero, end: Offset(1, 0)).animate(secondaryAnimation),
                child: child,
              )));
}

class RightToLeftWithFadeTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      SlideTransition(
          position: Tween<Offset>(begin: Offset(1, 0), end: Offset.zero).animate(animation),
          child: FadeTransition(
              opacity: animation,
              child: SlideTransition(
                position: Tween<Offset>(begin: Offset.zero, end: Offset(-1, 0)).animate(secondaryAnimation),
                child: child,
              )));
}

class UpToDownTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      SlideTransition(
        position: Tween<Offset>(begin: Offset(0, 1), end: Offset.zero).animate(animation),
        child: child,
      );
}

class DownToUpTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      SlideTransition(
        position: Tween<Offset>(begin: Offset(0, -1), end: Offset.zero).animate(animation),
        child: child,
      );
}

class ZoomTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      ScaleTransition(scale: animation, child: child);
}

class SizeTransitionBuilder extends CustomTransition {
  @override
  Widget buildTransitions(
    BuildContext context,
    Curve curve,
    Alignment alignment,
    Animation<double> animation,
    Animation<double> secondaryAnimation,
    Widget child,
  ) =>
      Align(
          alignment: Alignment.center,
          child: SizeTransition(
            sizeFactor: CurvedAnimation(parent: animation, curve: curve),
            child: child,
          ));
}
