import 'package:flutter/widgets.dart';
import 'package:yuro_core/yuro_core.dart';

import 'core/routing.dart';
import 'yuro_page_route.dart';

class YuroNavigatorObserver extends NavigatorObserver {
  final Routing routing;
  final RoutingCallback callback;

  YuroNavigatorObserver([this.routing, this.callback]);

  @override
  void didStopUserGesture() {
    super.didStopUserGesture();
    Yuro.log('YuroObserver | didStopUserGesture');
  }

  @override
  void didStartUserGesture(Route<dynamic> route, Route<dynamic> previousRoute) {
    super.didStartUserGesture(route, previousRoute);
    Yuro.log('YuroObserver | didStartUserGesture');
  }

  @override
  void didReplace({Route<dynamic> newRoute, Route<dynamic> oldRoute}) {
    super.didReplace(newRoute: newRoute, oldRoute: oldRoute);
    final newData = _RouteData.ofRoute(newRoute);
    final currentData = _RouteData.ofRoute(oldRoute);

    Yuro.log('YuroObserver | didReplace | old route: ${currentData.name}');
    Yuro.log('YuroObserver | didReplace | new route: ${newData.name}');

    Yuro.reference = newData.name;
    routing.update((obj) {
      if (newData.isYuroPageRoute) {
        obj.current = newData.name ?? '';
      }
      obj.args = newData.arguments;
      obj.route = newData.route;
      obj.previous = currentData.name;
      obj.removed = '';
      obj.isBack = false;
    });
    callback?.call(routing);
  }

  @override
  void didRemove(Route<dynamic> route, Route<dynamic> previousRoute) {
    super.didRemove(route, previousRoute);
    final newData = _RouteData.ofRoute(previousRoute);
    final currentData = _RouteData.ofRoute(route);

    Yuro.log('YuroObserver | didRemove | remove route: ${currentData.name}');

    routing.update((obj) {
      obj.route = newData.route;
      obj.previous = currentData.name;
      obj.removed = currentData.name;
      obj.isBack = false;
    });
    callback?.call(routing);
  }

  @override
  void didPop(Route<dynamic> route, Route<dynamic> previousRoute) {
    super.didPop(route, previousRoute);
    final newData = _RouteData.ofRoute(previousRoute);
    final currentData = _RouteData.ofRoute(route);

    if (currentData.isYuroPageRoute) {
      Yuro.log('YuroObserver | didPop | pop route: ${currentData.name}');
    }
    Yuro.reference = newData.name;
    routing.update((obj) {
      if (newData.isYuroPageRoute) {
        obj.current = newData.name ?? '';
      }
      obj.args = newData.arguments;
      obj.route = newData.route;
      obj.previous = currentData.name;
      obj.removed = '';
      obj.isBack = true;
    });
    callback?.call(routing);
  }

  @override
  void didPush(Route<dynamic> route, Route<dynamic> previousRoute) {
    super.didPush(route, previousRoute);
    final newData = _RouteData.ofRoute(route);
    final currentData = _RouteData.ofRoute(previousRoute);

    if (newData.isYuroPageRoute) {
      Yuro.log('YuroObserver | didPush | push route: ${newData.name}');
    }
    Yuro.reference = newData.name;
    routing.update((obj) {
      if (newData.route is PageRoute) {
        obj.current = newData.name ?? '';
      }
      obj.args = newData.arguments;
      obj.route = newData.route;
      obj.previous = currentData.name;
      obj.removed = '';
      obj.isBack = false;
    });
    callback?.call(routing);
  }
}

class _RouteData {
  final Route<dynamic> route;
  final String name;
  final dynamic arguments;
  final bool isYuroPageRoute;

  _RouteData({
    this.route,
    this.name,
    this.arguments,
    this.isYuroPageRoute,
  });

  factory _RouteData.ofRoute(Route route) {
    return _RouteData(
      route: route,
      name: _extractRouteName(route),
      arguments: route?.settings?.arguments,
      isYuroPageRoute: route is YuroPageRoute,
    );
  }
}

String _extractRouteName(Route route) {
  if (route?.settings?.name != null) {
    return route.settings.name;
  }
  if (route is YuroPageRoute) {
    return route.routeName;
  }
  return null;
}
