import 'package:flutter/widgets.dart';

import 'core/binding.dart';
import 'core/middleware.dart';
import 'core/transition.dart';

typedef YuroPageBuild = Widget Function();

class YuroPage {
  const YuroPage({
    @required this.name,
    @required this.build,
    this.parameter,
    this.settings,
    this.binding,
    this.middlewares,
    //
    this.alignment,
    this.curve,
    this.transition,
    this.customTransition,
    this.transitionDuration,
    //
    this.fullscreenDialog = false,
    this.opaque = true,
    this.maintainState = true,
    this.popGesture,
  })  : assert(name != null),
        assert(build != null),
        assert(fullscreenDialog != null),
        assert(maintainState != null);

  final String name;
  final YuroPageBuild build;
  final Map<String, String> parameter;
  final RouteSettings settings;

  final Binding binding;
  final List<YuroMiddleware> middlewares;

  final Alignment alignment;
  final Curve curve;
  final Transition transition;
  final CustomTransition customTransition;
  final Duration transitionDuration;

  final bool fullscreenDialog;
  final bool opaque;
  final bool maintainState;
  final bool popGesture;

  YuroPage copyWith({
    String name,
    YuroPageBuild build,
    Map<String, String> parameter,
    RouteSettings settings,
    Binding binding,
    List<YuroMiddleware> middlewares,
    Alignment alignment,
    Curve curve,
    Transition transition,
    CustomTransition customTransition,
    Duration transitionDuration,
    bool fullscreenDialog,
    bool opaque,
    bool maintainState,
    bool popGesture,
  }) {
    return YuroPage(
      name: name ?? this.name,
      build: build ?? this.build,
      parameter: parameter ?? this.parameter,
      settings: settings ?? this.settings,
      binding: binding ?? this.binding,
      middlewares: middlewares ?? this.middlewares,
      alignment: alignment ?? this.alignment,
      curve: curve ?? this.curve,
      transition: transition ?? this.transition,
      customTransition: customTransition ?? this.customTransition,
      transitionDuration: transitionDuration ?? this.transitionDuration,
      fullscreenDialog: fullscreenDialog ?? this.fullscreenDialog,
      opaque: opaque ?? this.opaque,
      maintainState: maintainState ?? this.maintainState,
      popGesture: popGesture ?? this.popGesture,
    );
  }
}
