import 'package:flutter/widgets.dart';

import 'package:yuro_core/yuro_core.dart';

import 'core/middleware.dart';

import 'yuro_page.dart';
import 'yuro_page_route.dart';
import 'yuro_route_ext.dart';

class YuroPageRedirect {
  YuroPageRedirect(
    this.settings, {
    this.unknownPage,
    this.onUnknownRoute,
    this.isUnknown = false,
  });

  RouteSettings settings;
  RouteFactory onUnknownRoute;
  YuroPage unknownPage;
  YuroPage page;
  bool isUnknown;

  YuroPageRoute route() {
    while (checkRedirect()) {}
    return isUnknown
        // 404界面路由
        ? unknownPage != null
            // YuroPage方式
            ? YuroPageRoute(
                build: unknownPage.build,
                parameter: unknownPage.parameter,
                settings: RouteSettings(name: settings.name, arguments: settings.arguments),
                binding: unknownPage.binding,
                middlewares: unknownPage.middlewares,
                alignment: unknownPage.alignment,
                curve: unknownPage.curve,
                transition: unknownPage.transition,
                customTransition: unknownPage.customTransition,
                transitionDuration: unknownPage.transitionDuration ?? Yuro.defaultTransitionDuration,
                fullscreenDialog: unknownPage.fullscreenDialog,
                opaque: unknownPage.opaque ?? Yuro.defaultOpaque,
                maintainState: unknownPage.maintainState,
                popGesture: unknownPage.popGesture ?? Yuro.defaultPopGesture,
              )
            // 默认方式
            : onUnknownRoute(settings)
        // 正常界面
        : YuroPageRoute(
            build: page.build,
            parameter: page.parameter,
            settings: RouteSettings(name: settings.name, arguments: settings.arguments),
            binding: page.binding,
            middlewares: page.middlewares,
            alignment: page.alignment,
            curve: page.curve,
            transition: page.transition,
            customTransition: page.customTransition,
            transitionDuration: page.transitionDuration ?? Yuro.defaultTransitionDuration,
            fullscreenDialog: page.fullscreenDialog,
            opaque: page.opaque ?? Yuro.defaultOpaque,
            maintainState: page.maintainState,
            popGesture: page.popGesture ?? Yuro.defaultPopGesture,
          );
  }

  bool checkRedirect() {
    var match = Yuro.routeTree.matchRoute(settings.name);
    Yuro.parameters = match?.parameters;
    // 判断是否匹配到路由
    if (match?.page == null) {
      isUnknown = true;
      return false;
    }
    // runRedirect ==> runPageCalled
    final runner = MiddlewareRunner(match.page.middlewares);
    final newSettings = runner.runRedirect(settings.name);
    if (newSettings == null) {
      page = runner.runPageCalled(match.page);
      return false;
    }
    settings = newSettings;
    return true;
  }
}
