import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:yuro_core/yuro_core.dart';

import 'core/binding.dart';
import 'core/route_tree.dart';
import 'core/routing.dart';
import 'core/transition.dart';
import 'yuro_page.dart';
import 'yuro_page_route.dart';

class _YuroRoute {
  GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

  final routing = Routing();

  Map<String, String> parameters;

  RouteTree routeTree;

  Curve defaultTransitionCurve = Curves.easeOutQuad;

  Transition defaultTransition;

  CustomTransition customTransition;

  Duration defaultTransitionDuration = Duration(milliseconds: 300);

  bool defaultPopGesture = Platform.isIOS;

  bool defaultOpaque = true;
}

extension YuroRouteExt on YuroInterface {
  static _YuroRoute _yuroRoute = _YuroRoute();

  void addPages(List<YuroPage> pages) => pages?.forEach((element) => routeTree.addRoute(element));

  GlobalKey<NavigatorState> get navigatorKey => _yuroRoute.navigatorKey;

  set navigatorKey(GlobalKey<NavigatorState> navigatorKey) => _yuroRoute.navigatorKey = navigatorKey;

  Routing get routing => _yuroRoute.routing;

  dynamic get arguments => routing.args;

  String get currentRoute => routing.current;

  Map<String, String> get parameters => _yuroRoute.parameters;

  set parameters(Map<String, String> parameters) => _yuroRoute.parameters = parameters;

  RouteTree get routeTree => _yuroRoute.routeTree ??= RouteTree();

  set routeTree(RouteTree routeTree) => _yuroRoute.routeTree = routeTree;

  CustomTransition get customTransition => _yuroRoute.customTransition;

  set customTransition(CustomTransition transition) => _yuroRoute.customTransition = transition;

  Transition get defaultTransition => _yuroRoute.defaultTransition;

  set defaultTransition(Transition transition) => _yuroRoute.defaultTransition = transition;

  Curve get defaultTransitionCurve => _yuroRoute.defaultTransitionCurve;

  set defaultTransitionCurve(Curve curve) => _yuroRoute.defaultTransitionCurve = curve;

  Duration get defaultTransitionDuration => _yuroRoute.defaultTransitionDuration;

  set defaultTransitionDuration(Duration duration) => _yuroRoute.defaultTransitionDuration = duration;

  bool get defaultPopGesture => _yuroRoute.defaultPopGesture;

  set defaultPopGesture(bool popGesture) => _yuroRoute.defaultPopGesture = popGesture;

  bool get defaultOpaque => _yuroRoute.defaultOpaque;

  set defaultOpaque(bool opaque) => _yuroRoute.defaultOpaque = opaque;

  Future<T> push<T>(
    Widget page, {
    Binding binding,
    dynamic arguments,
    bool opaque,
    Curve curve,
    Transition transition,
    Duration duration,
    bool fullscreenDialog = false,
    bool popGesture,
  }) {
    var routeName = '/${page.runtimeType.toString()}';
    return navigatorKey.currentState.push<T>(YuroPageRoute<T>(
      routeName: routeName,
      build: () => page,
      binding: binding,
      opaque: opaque ?? defaultOpaque,
      curve: curve ?? defaultTransitionCurve,
      transition: transition ?? defaultTransition,
      popGesture: popGesture ?? defaultPopGesture,
      settings: RouteSettings(arguments: arguments),
      fullscreenDialog: fullscreenDialog,
    ));
  }

  Future<T> pushNamed<T>(String page, {dynamic arguments}) {
    return navigatorKey.currentState.pushNamed<T>(page, arguments: arguments);
  }

  Future<T> pushReplacement<T>(
    Widget page, {
    dynamic result,
    Binding binding,
    dynamic arguments,
    bool opaque,
    Curve curve,
    Transition transition,
    Duration duration,
    bool fullscreenDialog = false,
    bool popGesture,
  }) {
    var routeName = '/${page.runtimeType.toString()}';
    return navigatorKey.currentState.pushReplacement<T, dynamic>(
        YuroPageRoute<T>(
          routeName: routeName,
          build: () => page,
          binding: binding,
          opaque: opaque ?? defaultOpaque,
          curve: curve ?? defaultTransitionCurve,
          transition: transition ?? defaultTransition,
          popGesture: popGesture ?? defaultPopGesture,
          settings: RouteSettings(arguments: arguments),
          fullscreenDialog: fullscreenDialog,
        ),
        result: result);
  }

  Future<T> pushReplacementNamed<T>(String routeName, {dynamic result, dynamic arguments}) {
    return navigatorKey.currentState.pushReplacementNamed<T, dynamic>(routeName, result: result, arguments: arguments);
  }

  Future<T> pushAndRemoveUntil<T>(
    Widget page, {
    RoutePredicate predicate,
    Binding binding,
    dynamic arguments,
    bool opaque,
    Curve curve,
    Transition transition,
    Duration duration,
    bool fullscreenDialog = false,
    bool popGesture,
  }) {
    var routeName = '/${page.runtimeType.toString()}';
    return navigatorKey.currentState.pushAndRemoveUntil<T>(
        YuroPageRoute<T>(
          routeName: routeName,
          build: () => page,
          binding: binding,
          opaque: opaque ?? defaultOpaque,
          curve: curve ?? defaultTransitionCurve,
          transition: transition ?? defaultTransition,
          popGesture: popGesture ?? defaultPopGesture,
          settings: RouteSettings(arguments: arguments),
          fullscreenDialog: fullscreenDialog,
        ),
        predicate ?? (_) => false);
  }

  Future<T> pushNamedAndRemoveUntil<T>(String newRouteName, {RoutePredicate predicate, dynamic arguments}) {
    return navigatorKey.currentState.pushNamedAndRemoveUntil<T>(newRouteName, predicate ?? (_) => false, arguments: arguments);
  }

  void pop<T>([T result]) => navigatorKey.currentState.pop<T>(result);

  void popUntil(RoutePredicate predicate) => navigatorKey.currentState.popUntil(predicate);

  Future<T> popAndPushNamed<T>(String routeName, {dynamic result, dynamic arguments}) {
    return navigatorKey.currentState.popAndPushNamed<T, dynamic>(routeName, result: result, arguments: arguments);
  }

  void removeRoute(Route<dynamic> route) => navigatorKey.currentState.removeRoute(route);

  void removeRouteBelow(Route<dynamic> anchorRoute) => navigatorKey.currentState.removeRouteBelow(anchorRoute);

  void replace<T>({@required Route<dynamic> oldRoute, @required Route<T> newRoute}) {
    return navigatorKey.currentState.replace(oldRoute: oldRoute, newRoute: newRoute);
  }

  void replaceRouteBelow<T>({@required Route<dynamic> anchorRoute, @required Route<T> newRoute}) {
    return navigatorKey.currentState.replaceRouteBelow(anchorRoute: anchorRoute, newRoute: newRoute);
  }

  bool canPop() => navigatorKey.currentState.canPop();
}
